/////////////////////////////////////////////////////////////////
/////////////THE MODEL WITH FINANCIAL FRICTIONS /////////////////
/////////////////////////////////////////////////////////////////


// This code introduces financial frictions as in Bernanke, Gertler and Gilchrist (1999) in the Smets and Wouters model.
// The code runs on the last version of Dynare (Dynare 4.4.0)
// Code written by Rossana Merola (rossana.merola@esri.ie; rmerola@tcd.ie; rossana.merola@gmail.com)


// I. ENDOGENOUS VARIABLES

var  sobs labobs robs pinfobs dy dc dinve dw ewma epinfma zcapf rkf kf pkf cf invef yf labf wf rrf mc zcap rk k pk c inve y lab pinf w r a  b b1 g qs ms spinf sw kpf kp 
     ef f nw  prem   eff ff nwf premf ygap lev ;


// II. EXOGENOUS VARIABLES
varexo  ea eb eb1 eg  eqs  em  epinf ew ;  


// III. PARAMETERS

parameters curvw cgy curvp constelab constepinf constebeta cmaw cmap calfa czcap csadjcost ctou csigma chabb cfc 
cindw cprobw cindp cprobp csigl clandaw  crpi crdy cry crr crhoa crhoas crhob crhob1 crhog crhols crhoqs crhoms crhopinf crhow  
ctrend cg ceffe clev cv elast ;

// 1. Fixed parameters
ctou=.025;
clandaw=1.5;
cg=0.18;
curvp=10;
curvw=10;

// 2.Estimated parameters initialisation
calfa=.24;
cgamma_init=1.004;
cbeta=.9995;
csigma=1.5;
cpie_init=1.005;
cfc=1.5;
cgy=0.51;
csadjcost= 6.0144;
chabb=    0.6361;    
cprobw=   0.8087;
csigl=    1.9423;
cprobp=   0.6;
cindw=    0.3243;
cindp=    0.47;
czcap=    0.2696;
crpi=     1.488;
crr=      0.8762;
cry=      0.0593;
crdy=     0.2347;
crhoa=    0.9977;
crhob=    0.5799;
crhob1=    0.5799;
crhog=    0.9957;
crhols=   0.9928;
crhoqs=   0.7165;
crhoas=1; 
crhoms=0;
crhopinf=0;
crhow=0;
cmap = 0;
cmaw  = 0;

// 3. Derived from steady state
clandap=cfc;
cbetabar=cbeta*cgamma_init^(-csigma);
cr_init=cpie_init/(cbeta*cgamma_init^(-csigma));
crk=(cbeta^(-1))*(cgamma_init^csigma) - (1-ctou);
cw = (calfa^calfa*(1-calfa)^(1-calfa)/(clandap*crk^calfa))^(1/(1-calfa));
cikbar=(1-(1-ctou)/cgamma_init);
cik=(1-(1-ctou)/cgamma_init)*cgamma_init;
clk=((1-calfa)/calfa)*(crk/cw);
cky=cfc*(clk)^(calfa-1);
ciy=cik*cky;
ccy=1-cg-cik*cky;
crkky=crk*cky;
cwhlc=(1/clandaw)*(1-calfa)/calfa*crk*cky/ccy;
cwly=1-crk*cky;
ctrend=(cgamma_init-1)*100;
conster=(cr_init-1)*100;
constepinf=(cpie_init-1)*100;
constelab=0;

// 4. Financial accelerator parameters
ceffe=1.0129;
clev=1.7;
cv=0.99;
elast=0.05;

// IV. MODEL

model(linear); 

# cpie=1+constepinf/100;
# cgamma=1+ctrend/100 ;
# cbeta=1/(1+constebeta/100);
# clandap=cfc;
# cbetabar=cbeta*cgamma^(-csigma);
# cr=cpie/(cbeta*cgamma^(-csigma));
# crk=(cbeta^(-1))*(cgamma^csigma) - (1-ctou);
# cw = (calfa^calfa*(1-calfa)^(1-calfa)/(clandap*crk^calfa))^(1/(1-calfa));
# cikbar=(1-(1-ctou)/cgamma);
# cik=(1-(1-ctou)/cgamma)*cgamma;
# clk=((1-calfa)/calfa)*(crk/cw);
# cky=cfc*(clk)^(calfa-1);
# ciy=cik*cky;
# ccy=1-cg-cik*cky;
# crkky=crk*cky;
# cwhlc=(1/clandaw)*(1-calfa)/calfa*crk*cky/ccy;
# cwly=1-crk*cky;
# conster=(cr-1)*100;

// 1. The flexible economy

	      0*(1-calfa)*a + 1*a =  calfa*rkf+(1-calfa)*(wf)  ;
	      zcapf =  (1/(czcap/(1-czcap)))* rkf  ;
	      rkf =  (wf)+labf-kf ;
	      kf =  kpf(-1)+zcapf ;
	      invef = (1/(1+cbetabar*cgamma))* (  invef(-1) + cbetabar*cgamma*invef(1)+(1/(cgamma^2*csadjcost))*pkf ) +qs ;
              cf = (chabb/cgamma)/(1+chabb/cgamma)*cf(-1) + (1/(1+chabb/cgamma))*cf(+1) +((csigma-1)*cwhlc/(csigma*(1+chabb/cgamma)))*(labf-labf(+1)) - (1-chabb/cgamma)/(csigma*(1+chabb/cgamma))*(rrf)+1*b1 ;
	      yf = ccy*cf+ciy*invef+g  +  crkky*zcapf ;
	      yf = cfc*( calfa*kf+(1-calfa)*labf +a );
	      wf = csigl*labf 	+(1/(1-chabb/cgamma))*cf - (chabb/cgamma)/(1-chabb/cgamma)*cf(-1) ;
	      kpf =  (1-cikbar)*kpf(-1)+(cikbar)*invef + (cikbar)*(cgamma^2*csadjcost)*qs ;
              ff+pkf(-1)= (crk/ceffe)*rkf + ((1-ctou)/ceffe)*pkf ;
              (1/(cv*ceffe))*nwf =clev*ff-elast*(clev-1)*(pkf(-1)+kf(-1))-(clev-1)*(rrf(-1))+(elast*(clev-1)+1)*nwf(-1);
              eff=(rrf)+ (1/((1-chabb/cgamma)/(csigma*(1+chabb/cgamma))))*b + elast*(pkf(1)+kf(1)-nwf(1));
              eff=ff(+1);
              premf =eff-rrf -(1/((1-chabb/cgamma)/(csigma*(1+chabb/cgamma))))*b;  

// 2. The economy with sticky prices and sticky wages

	      mc =  calfa*rk+(1-calfa)*(w) - 1*a - 0*(1-calfa)*a ;
	      zcap =  (1/(czcap/(1-czcap)))* rk ;
	      rk =  w+lab-k ;
	      k =  kp(-1)+zcap ;
	      inve = (1/(1+cbetabar*cgamma))* (  inve(-1) + cbetabar*cgamma*inve(1)+(1/(cgamma^2*csadjcost))*pk ) +qs ;
              c = (chabb/cgamma)/(1+chabb/cgamma)*c(-1) + (1/(1+chabb/cgamma))*c(+1) +((csigma-1)*cwhlc/(csigma*(1+chabb/cgamma)))*(lab-lab(+1)) - (1-chabb/cgamma)/(csigma*(1+chabb/cgamma))*(r-pinf(+1)) + 1*b1 ;
	      y = ccy*c+ciy*inve+g  +  1*crkky*zcap ;
	      y = cfc*( calfa*k+(1-calfa)*lab +a );
	      pinf =  (1/(1+cbetabar*cgamma*cindp)) * ( cbetabar*cgamma*pinf(1) +cindp*pinf(-1) 
               +((1-cprobp)*(1-cbetabar*cgamma*cprobp)/cprobp)/((cfc-1)*curvp+1)*(mc)  )  + spinf ; 
	      w =  (1/(1+cbetabar*cgamma))*w(-1)
               +(cbetabar*cgamma/(1+cbetabar*cgamma))*w(1)
               +(cindw/(1+cbetabar*cgamma))*pinf(-1)
               -(1+cbetabar*cgamma*cindw)/(1+cbetabar*cgamma)*pinf
               +(cbetabar*cgamma)/(1+cbetabar*cgamma)*pinf(1)
               +(1-cprobw)*(1-cbetabar*cgamma*cprobw)/((1+cbetabar*cgamma)*cprobw)*(1/((clandaw-1)*curvw+1))*
               (csigl*lab + (1/(1-chabb/cgamma))*c - ((chabb/cgamma)/(1-chabb/cgamma))*c(-1) -w) 
               + 1*sw ;
	     f+pk(-1)= (crk/ceffe)*rk + ((1-ctou)/ceffe)*pk ;
             (1/(cv*ceffe))*nw =clev*f-elast*(clev-1)*(pk(-1)+k(-1))-(clev-1)*(r(-1)-pinf)+(elast*(clev-1)+1)*nw(-1);//real FA
             ef=(r-pinf(1))+ (1/((1-chabb/cgamma)/(csigma*(1+chabb/cgamma))))*b + elast*(pk(1)+k(1)-nw(1));
             ef=f(+1);
             prem =ef-(r-pinf(1)) - (1/((1-chabb/cgamma)/(csigma*(1+chabb/cgamma))))*b ; 
             lev=pk+k-nw;


// 3. Taylor rule

             r =  crpi*(1-crr)*pinf+cry*(1-crr)*(y-yf)+crdy*(y-yf-y(-1)+yf(-1))+crr*r(-1)+ms  ;
  
// 4. Output gap - definition

             ygap=y-yf;
            
// 5. AR(1) shocks

	      a = crhoa*a(-1)  + ea;
	      b = crhob*b(-1) + eb; 
              b1 = crhob1*b1(-1) + eb1;
	      g = crhog*(g(-1)) + eg + cgy*ea;
	      qs = crhoqs*qs(-1) + eqs;
	      ms = crhoms*ms(-1) + em;
	      spinf = crhopinf*spinf(-1) + epinfma - cmap*epinfma(-1);
	      epinfma=epinf;
	      sw = crhow*sw(-1) + ewma - cmaw*ewma(-1) ;
	      ewma=ew; 
	      kp =  (1-cikbar)*kp(-1)+cikbar*inve + cikbar*cgamma^2*csadjcost*qs ;



// 6. Measurment equations

             dy=y-y(-1)+ctrend;
             dc=c-c(-1)+ctrend;
             dinve=inve-inve(-1)+ctrend;
             dw=w-w(-1)+ctrend;
             pinfobs = 1*(pinf) + constepinf;
             robs =    1*(r) + conster;
             labobs = lab + constelab;
             sobs =  prem ;

end; 



// V. SHOCKS

shocks;
var ea;
stderr 0.4618;
var eb;
stderr  1.8513;
var eb1;
stderr 1.8513;
var eg;
stderr 0.6090;
var eqs;
stderr 0.6017;
var em;
stderr 0.2397;
var epinf;
stderr 0.1455;
var ew;
stderr 0.2089;
end;


// VI. ESTIMATION

estimated_params;

// Definition: 
// PARAM NAME, INITVAL, LB, UB, PRIOR_SHAPE, PRIOR_P1, PRIOR_P2, PRIOR_P3, PRIOR_P4, JSCALE
// PRIOR_SHAPE: BETA_PDF, GAMMA_PDF, NORMAL_PDF, INV_GAMMA_PDF

// The definition of priors closely follows Smest and Wouters (2007). As for the additional parameters linked to the financial accelerator mechanism:
// (i) for the leverage ratio (lev), I look at the historical data reported in Gilchrist, Ortiz and Zakrajsek (2009)
// (ii) for the elasticity (elast), I follow Carlstrom, Fuerst, Ortiz and Paustian (2012)


stderr ea,0.4582,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr eb,1.3692,0.025,5,INV_GAMMA_PDF,0.1,2;
stderr eb1,1.045,0.025,5,INV_GAMMA_PDF,0.1,2;
stderr eg,0.4972,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr eqs,0.3312,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr em,0.2381,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr epinf,0.2183,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr ew,0.2749,0.01,3,INV_GAMMA_PDF,0.1,2;

crhoa,.9527 ,.01,.9999,BETA_PDF,0.5,0.20;
crhob,.7296,.01,.9999,BETA_PDF,0.5,0.20;
crhob1,.4467,.01,.9999,BETA_PDF,0.5,0.20;
crhog,.9786,.01,.9999,BETA_PDF,0.5,0.20;
crhoqs,.9748,.01,.9999,BETA_PDF,0.5,0.20;
crhoms,.2048,.01,.9999,BETA_PDF,0.5,0.20;
crhopinf,.93,.01,.9999,BETA_PDF,0.5,0.20;  
crhow,.9208,.001,.9999,BETA_PDF,0.5,0.20;
cmap,.915,0.01,.9999,BETA_PDF,0.5,0.2;  
cmaw,.9027,0.01,.9999,BETA_PDF,0.5,0.2;

csadjcost,4.8115,2,15,NORMAL_PDF,4.5,1.5;
csigma,1.5036,0.25,3,NORMAL_PDF,1.50,0.375;
chabb,0.5748,0.001,0.99,BETA_PDF,0.7,0.1;    
cprobw,0.9164,0.3,0.95,BETA_PDF,0.5,0.1;
csigl,1.554,0.25,10,NORMAL_PDF,2,0.75;
cprobp,0.8045,0.5,0.95,BETA_PDF,0.5,0.10;
cindw,0.5543,0.01,0.99,BETA_PDF,0.5,0.15;
cindp,0.2597,0.01,0.99,BETA_PDF,0.5,0.15;
czcap,0.875,0.01,1,BETA_PDF,0.5,0.15;
cfc,1.6138,1.0,3,NORMAL_PDF,1.25,0.125;
crpi,1.3611,1.0,3,NORMAL_PDF,1.5,0.25;
crr,0.7832,0.5,0.975,BETA_PDF,0.75,0.10;
cry,0.0352,0.001,0.5,NORMAL_PDF,0.125,0.05;
crdy,0.2665,0.001,0.5,NORMAL_PDF,0.125,0.05;
constepinf,0.7698,0.01,2.0,GAMMA_PDF,0.62,0.2; 
constebeta,0.1025,0.01,2.0,GAMMA_PDF,0.25,0.1;   
constelab,5.8625,-10.0,10.0,NORMAL_PDF,0.0,2.0;
ctrend,0.4083,0.1,0.8,NORMAL_PDF,0.4,0.10;
cgy,0.5231,0.01,2.0,NORMAL_PDF,0.5,0.25;
calfa,0.1714,0.01,1.0,NORMAL_PDF,0.3,0.05;
clev,1.625,1,4,NORMAL_PDF,1.7, 0.2;  
elast,0.028,0.01, 0.5, NORMAL_PDF,0.05,0.02;

end;

varobs dy dc dinve labobs pinfobs dw robs sobs;

estimation(optim=('MaxIter',200),datafile=USdata2012,mode_compute=1,first_obs=71,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2);
//estimation(optim=('MaxIter',200),datafile=USdata2012,mode_compute=0,mode_file=SW_FA_Dynare_mode, first_obs=71,presample=4,lik_init=2,prefilter=0,mh_replic=0,mh_nblocks=2,mh_jscale=0.20,mh_drop=0.2, filtered_vars, smoother);
stoch_simul(irf=20, conditional_variance_decomposition=[1, 10, 40]) dy dc dinve robs pinfobs dw labobs sobs;
//shock_decomposition dy;
